﻿/******************************************************************************
Send bug reports to: philip@spigelmire.com

Copyright (c) 2010, Weatherite Corporation
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE 
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
******************************************************************************/

using System;

namespace Opto22Library
{
    public static class O22SnapIoHelper
    {
        public static string GetModuleName(UInt32 ModuleType)
        {
            switch (ModuleType)
            {
                case 0x00000000: return "4-ch digital or empty";
                case 0x00000004: return "SNAP-AICTD";
                case 0x00000009: return "SNAP-AITM-2";
                case 0x0000000A: return "SNAP-AIPM";
                case 0x0000000B: return "SNAP-AILC";
                case 0x0000000C: return "SNAP-AILC-2";
                case 0x00000010: return "SNAP-AIRTD";
                case 0x00000012: return "SNAP-AIV";
                case 0x00000020: return "SNAP-AITM-i";
                case 0x00000021: return "SNAP-AITM2-i";
                case 0x00000022: return "SNAP-AIMA-i";
                case 0x00000023: return "SNAP-AIV-i";
                case 0x00000024: return "SNAP-AIV2-i";
                case 0x00000025: return "SNAP-pH/ORP";
                case 0x00000026: return "SNAP-AIMA-iSRC(-FM)";
                case 0x00000027: return "SNAP-AIMA2-i";
                case 0x00000028: return "SNAP-AIARMS-I(-FM)";
                case 0x00000029: return "SNAP-AIVRMS-i(-FM)";
                case 0x00000040: return "SNAP-AIMA-4";
                case 0x00000041: return "SNAP-AIV-4";
                case 0x00000042: return "SNAP-AICTD-4";
                case 0x00000043: return "SNAP-AIR40K-4";
                case 0x00000044: return "SNAP-AIMV-4";
                case 0x00000045: return "SNAP-AIMV2-4";
                case 0x00000049: return "SNAP-AIPM-3";
                case 0x0000004A: return "SNAP-AIMA-8";
                case 0x0000004B: return "SNAP-AIV-8";
                case 0x0000004C: return "SNAP-AICTD-8";
                case 0x0000004D: return "SNAP-AIMA-32(-FM)";
                case 0x0000004E: return "SNAP-AIV-32";
                case 0x0000004F: return "SNAP-AITM-8";
                case 0x00000064: return "SNAP-AIMA";
                case 0x00000066: return "SNAP-AITM";
                case 0x00000069: return "SNAP-AIRATE";
                case 0x00000070: return "SNAP-AIVRMS";
                case 0x00000071: return "SNAP-AIARMS";
                case 0x000000A3: return "SNAP-AOA23";
                case 0x000000A5: return "SNAP-AOV25";
                case 0x000000A7: return "SNAP-AOV27";
                case 0x000000A8: return "SNAP-AOA28";
                case 0x000000A9: return "SNAP-AOD29";
                case 0x000000B3: return "SNAP-AOA23-iSRC(-FM)";
                case 0x000000D0: return "SNAP-PID-V";
                case 0x000000E0: return "SNAP-IDC-32(-FM)";
                case 0x000000E1: return "SNAP-ODC-32-SRC(-FM)";
                case 0x000000E2: return "SNAP-ODC-32-SNK(-FM)";
                case 0x000000E3: return "SNAP-IAC-A-16";
                case 0x000000E4: return "SNAP-IAC-16";
                case 0x000000E5: return "SNAP-IDC-16";
                case 0x000000E6: return "SNAP-IDC-32N";
                case 0x000000E7: return "SNAP-IAC-K-16";
                case 0x000000E8: return "SNAP-IDC-HT-16";
                case 0x000000F0: return "SNAP-SCM-232";
                case 0x000000F1: return "SNAP-SCM-485(-422)";
                case 0x000000F6: return "SNAP-SCM-PROFI";
                case 0x000000F8: return "SNAP-SCM-MCH16";
                case 0x000000F9: return "SNAP-SCM-W2";
                default:
                    return "Unkown";
            }
        }

        public static byte[] GetBytes(bool value)
        {
            byte[] rv = System.BitConverter.GetBytes(value);
            return rv;
        }

        public static byte[] GetBytes(UInt16 value)
        {
            byte[] rv = System.BitConverter.GetBytes(value);
            Array.Reverse(rv);
            return rv;
        }

        public static byte[] GetBytes(UInt32 value)
        {
            byte [] rv = System.BitConverter.GetBytes(value);
            Array.Reverse(rv);
            return rv;
        }

        public static byte[] GetBytes(UInt64 value)
        {
            byte[] rv = System.BitConverter.GetBytes(value);
            Array.Reverse(rv);
            return rv;
        }

        public static byte[] GetBytes(Single value)
        {
            byte[] rv = System.BitConverter.GetBytes(value);
            Array.Reverse(rv);
            return rv;
        }

        public static bool ToBoolean(byte[] value, int pos)
        {
            return System.BitConverter.ToBoolean(value, pos);
        }

        public static UInt16 ToUInt16x(this byte[] value, int pos)
        {
            return ToUInt16(value, pos);
        }

        public static UInt16 ToUInt16(byte[] value, int pos)
        {
            byte[] temp = new byte[sizeof(UInt16)];
            Array.Copy(value, pos, temp, 0, sizeof(UInt16));
            Array.Reverse(temp);
            return System.BitConverter.ToUInt16(temp, 0);
        }

        public static UInt32 ToUInt32x(this byte[] value, int pos)
        {
            return ToUInt32(value, pos);
        }

        public static UInt32 ToUInt32(byte[] value, int pos)
        {
            byte[] temp = new byte[sizeof(UInt32)];
            Array.Copy(value, pos, temp, 0, sizeof(UInt32));
            Array.Reverse(temp);
            return System.BitConverter.ToUInt32(temp, 0);
        }

        public static UInt64 ToUInt64(byte[] value, int pos)
        {
            byte[] temp = new byte[sizeof(UInt64)];
            Array.Copy(value, pos, temp, 0, sizeof(UInt64));
            Array.Reverse(temp);
            return System.BitConverter.ToUInt64(temp, 0);
        }

        public static Single ToSingle(byte[] value, int pos)
        {
            byte[] temp = new byte[sizeof(Single)];
            Array.Copy(value, pos, temp, 0, sizeof(Single));
            Array.Reverse(temp);
            return System.BitConverter.ToSingle(temp, 0);
        }
    }
}
